"""Tests for `acdh_tei_pyutils.utils` module."""

import unittest

from acdh_tei_pyutils.tei import TeiReader
from acdh_tei_pyutils.utils import make_bibl_label


class TestTeiEnricher(unittest.TestCase):
    def test_001(self):
        doc = TeiReader("tests/listbibl_test.xml")
        for x in doc.any_xpath(".//tei:biblStruct[@xml:id]"):
            label = make_bibl_label(x, max_title_length=50)
            n = x.attrib["n"]
            self.assertEqual(label, n)



#!/usr/bin/env python

"""Tests for `acdh_tei_pyutils.tei` module."""

import glob
import unittest
import lxml.etree as ET
from acdh_tei_pyutils.tei import NER_TAG_MAP, TeiReader, TeiEnricher, HandleAlreadyExist
from acdh_tei_pyutils.utils import (
    normalize_string,
    make_entity_label,
    get_birth_death_year,
    check_for_hash,
    add_graphic_url_to_pb,
    extract_fulltext,
    get_xmlid,
)


FILES = sorted(glob.glob("./acdh_tei_pyutils/files/*.xml", recursive=False))

XSL = glob.glob("./acdh_tei_pyutils/files/*.xsl", recursive=False)[0]

IDS_AND_SO = [
    ["base_value", "id_value", "prev_value", "next_value"],
    [None, None, None, None],
    ["base_value", "id_value", None, None],
    ["base_value/", "id_value", "prev_value", "next_value"],
]
FULL_IDS = ["base_value/id_value", None, "base_value/id_value", "base_value/id_value"]


class TestTeiEnricher(unittest.TestCase):
    """Tests for `acdh_tei_pyutils.tei.TeiEnricher` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_init(self):
        for x in FILES:
            doc = TeiEnricher(xml=x)
            self.assertIsInstance(doc.return_string(), str)

    def test_002_add_context(self):  # ToDo: check for ordering of files
        dummy_data = zip(FILES, IDS_AND_SO, FULL_IDS)
        for x in dummy_data:
            doc = TeiEnricher((x[0]))
            doc.add_base_and_id(*x[1])
            full_id = doc.get_full_id()
            self.assertEqual(full_id, x[2])

    def test_003_handle_exist(self):
        hdl_doc = TeiEnricher(xml="./acdh_tei_pyutils/files/tei.xml")
        self.assertIn("http", hdl_doc.handle_exist())
        hdl_no = TeiEnricher(xml="./acdh_tei_pyutils/files/tei_no_id.xml")
        self.assertIsNone(hdl_no.handle_exist())

    def test_004_add_handle(self):
        hdl_doc = TeiEnricher(xml="./acdh_tei_pyutils/files/tei.xml")
        self.assertRaises(HandleAlreadyExist, lambda: hdl_doc.add_handle("1234/5432"))
        hdl_no = TeiEnricher(xml="./acdh_tei_pyutils/files/tei_no_id.xml")
        handle_node = hdl_no.add_handle("1234/5432")
        self.assertEqual(handle_node.text, "1234/5432")
        self.assertEqual(hdl_no.handle_exist(), "1234/5432")


class TestTEIReader(unittest.TestCase):
    """Tests for `acdh_tei_pyutils.tei.TeiReader` class."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_001_ner_mapping(self):
        self.assertIsInstance(NER_TAG_MAP, dict)

    def test_002_parsing_from_file(self):
        for x in FILES:
            doc = TeiReader(xml=x)
            parent_node = doc.any_xpath(any_xpath="//tei:body")[0]
            ne_list = doc.extract_ne_elements(parent_node)
            self.assertIsInstance(ne_list, list)

    def test_003_extract_ner_list(self):
        doc = TeiReader(xml=FILES[0])
        ne_list = doc.get_text_nes_list()
        self.assertIsInstance(ne_list, list)
        self.assertTrue(len(ne_list), 2)

    def test_004_check_ner_list(self):
        doc = TeiReader(xml=FILES[0])
        ne_list = doc.get_text_nes_list()
        for y in ne_list:
            print(y)
            if y["ner_dicts"]:
                self.assertTrue(
                    y["ner_dicts"][0]["text"] in ["Prag", "Broschüre", "Böhmen"]
                )

    def test_005_ner_offsets(self):
        doc = TeiReader(xml=FILES[0])
        ne_offsets = doc.extract_ne_offsets()
        print(ne_offsets[2])
        self.assertIsInstance(ne_offsets, list)

    def test_006_markup_cleanup(self):
        doc = TeiReader(xml=FILES[0])
        ent_list = doc.get_elements()
        self.assertTrue("{http://www.tei-c.org/ns/1.0}unclear" in ent_list)
        doc = TeiReader(xml=FILES[0], xsl=XSL)
        ent_list = doc.get_elements()
        self.assertFalse("{http://www.tei-c.org/ns/1.0}unclear" in ent_list)

    def test_007_normalize_string(self):
        string = """\n\nhallo
mein schatz ich liebe    dich
    du bist         die einzige für mich
        """
        normalized = normalize_string(string)
        self.assertTrue("\n" not in normalized)

    def test_008_make_pers_name_labels(self):
        test_names = """
<TEI xmlns="http://www.tei-c.org/ns/1.0" >
    <back>
        <persName>
            <forename>Johann</forename>
            <surname>Thayer</surname>
        </persName>
        <orgName>orgName</orgName>
        <placeName>PlaceName</placeName>
        <persName>
            <forename>Josef</forename>
            <forename type="taken" subtype="later">Karl</forename>
            <surname>Crcil</surname>
            <surname type="taken" subtype="later">Graf</surname>
        </persName>
        <persName></persName>
        <persName>
            <forename>NurVorname</forename>
        </persName>
        <persName xml:lang="de">
            <surname>NurNachname</surname>
            <forename></forename>
        </persName>
    </back>
</TEI>
"""
        labels = [
            "Thayer, Johann",
            "orgName",
            "PlaceName",
            "Crcil, Josef Karl",
            "no label provided",
            "NurVorname",
            "NurNachname",
        ]
        doc = TeiReader(test_names)
        for i, x in enumerate(doc.any_xpath(".//tei:back/*")):
            label, lang = make_entity_label(x, default_lang="en")
            self.assertEqual(label, labels[i])
        self.assertEqual(lang, "de")

    def test_009_birth_death_year(self):
        test_str = """
<listPerson xmlns="http://www.tei-c.org/ns/1.0">
    <person>
        <birth when-iso="1983-08-04"></birth>
    </person>
    <person>
        <death when="1982-08-04"></death>
    </person>
    <person>
        <death when="foo"></death>
        <birth when-iso="bar"></birth>
    </person>
    </listPerson>
"""
        doc = TeiEnricher(test_str)
        results = set()
        for x in doc.any_xpath(".//tei:person"):
            birth_year = get_birth_death_year(x, xpath_part="@when-iso")
            death_year = get_birth_death_year(x, birth=False)
            results.add(birth_year)
            results.add(death_year)
        self.assertTrue(1982 in results)
        self.assertTrue(1983 in results)
        self.assertTrue(None in results)

    def test_010_check_for_hash(self):
        test_values = ["#hansi", "hansi"]
        for x in test_values:
            checked = check_for_hash(x)
            self.assertEqual(checked, "hansi")

    def test_011_pb_facs_uris(self):
        test_str = """
<TEI xmlns="http://www.tei-c.org/ns/1.0">
    <facsimile>
        <surfaceGrp>
            <surface type="recto" xml:id="D_000002-003-000-facs001-l001-p001">
                <graphic ana="status:checked" source="wienbibliothek"
                url="https://www.digital.wienbibliothek.at/wbrobv02/i3f/v21/2540032/full/full/0/default.jpg"/>
            </surface>
            <surface type="verso" xml:id="D_000002-003-000-facs001-l001-p002">
                <graphic ana="status:checked" source="wienbibliothek"
                url="https://www.digital.wienbibliothek.at/wbrobv02/i3f/v21/2540034/full/full/0/default.jpg"/>
            </surface>
        </surfaceGrp>
    </facsimile>
    <pb facs="#D_000002-003-000-facs001-l001-p001" n="1"/>
    <pb facs="#D_000002-003-000-facs001-l001-p002" n="2"/>
    <pb facs="#D_000002-003-000-facs001-l001-p003" n="2"/>
</TEI>
"""
        doc = TeiReader(test_str)
        new_doc = add_graphic_url_to_pb(doc)
        graphic_urls = new_doc.any_xpath(".//tei:graphic/@url")
        pb_urls = new_doc.any_xpath(".//tei:pb/@url")
        for x in pb_urls:
            self.assertTrue(x in graphic_urls)

    def test_012_extract_fulltext(self):
        test_str = """
<TEI xmlns="http://www.tei-c.org/ns/1.0">
    <body>
    <lb n="N001"/>Durchleücht<supplied>i</supplied>ger Curfü<supplied>r</supplied>st,
    mein herzallerlibster <choice><expan>herr</expan><abbr>h</abbr></choice> bruder
      <lb n="N002"/>Ich kom nuhr in eil, dero Liebden zu participiren die victori,
      <lb n="N003"/>so <choice><expan>Prinz</expan><abbr>P</abbr></choice> Eugen
      in Italien gehabt. Ist sehr scharf
      <lb n="N004"/>her gangen, der <choice><expan>Prinz</expan><abbr>Pz</abbr></choice>
      ist durch den hals geschoßen, <choice><expan>Prinz</expan><abbr>Pr</abbr></choice> Joseph
      <lb n="N005"/>auch durch die wang, aber bede gottlob ohn gefahr,
    </body>
</TEI>
"""
        doc = TeiReader(test_str)
        body = doc.any_xpath(".//tei:body")[0]
        ft = extract_fulltext(body)
        self.assertTrue("PrinzPz" in ft)
        ft = extract_fulltext(body, tag_blacklist=["{http://www.tei-c.org/ns/1.0}abbr"])
        self.assertFalse("PrinzPz" in ft)

    def test_013_get_xmlid(self):
        node = ET.Element("{http://www.tei-c.org/ns/1.0}person")
        node.attrib["{http://www.w3.org/XML/1998/namespace}id"] = "foo"
        xml_id = get_xmlid(node)
        self.assertEqual("foo", xml_id)



import lxml.etree as ET
from itertools import tee, islice, chain
from typing import Union

from acdh_tei_pyutils.tei import TeiReader

nsmap = {
    "tei": "http://www.tei-c.org/ns/1.0",
    "xml": "http://www.w3.org/XML/1998/namespace",
}


def get_xmlid(element: ET.Element) -> str:
    """returns an @xml:id of the given node"""
    return element.attrib["{http://www.w3.org/XML/1998/namespace}id"]


def crate_tag_whitelist(element: ET.Element, tag_blacklist: list) -> list:
    """lists all unique elements from a given node and returns only those not in the given blacklist"""
    tags = list(
        set([x.tag for x in element.iter(tag=ET.Element) if x.tag not in tag_blacklist])
    )
    return tags


def extract_fulltext(root_node: ET.Element, tag_blacklist: list = []) -> str:
    """extracts all fulltext from given element and its children, except from blacklisted elements"""
    tags = crate_tag_whitelist(root_node, tag_blacklist)
    full_text = " ".join("".join(root_node.itertext(*tags)).split())
    return full_text


def check_for_hash(value: str) -> str:
    """checks if value starts with '#' and if so removes the '#' from the returned value"""
    if value.startswith("#"):
        return value[1:]
    else:
        return value


def add_graphic_url_to_pb(doc: TeiReader) -> TeiReader:
    """writes url attributes into tei:pb elements fetched from matching tei:surface//tei:graphic[1] elements"""
    for x in doc.any_xpath(".//tei:pb[@facs]"):
        facs_id = check_for_hash(x.attrib["facs"])
        xpath_expr = f'.//tei:surface[@xml:id="{facs_id}"]//tei:graphic[1]/@url'
        try:
            facs_url = doc.any_xpath(xpath_expr)[0]
        except IndexError:
            continue
        x.attrib["url"] = facs_url
    return doc


def get_birth_death_year(
    person_node: ET.Element, xpath_part: str = "@when", birth: bool = True
) -> Union[int, bool]:
    """tries to extract birth and death years from person nodes and returns either None or the year as Integer"""
    if birth:
        year_xpath = f"./tei:birth/{xpath_part}"
    else:
        year_xpath = f"./tei:death/{xpath_part}"
    try:
        date_str = person_node.xpath(
            year_xpath, namespaces={"tei": "http://www.tei-c.org/ns/1.0"}
        )[0]
    except IndexError:
        return None
    year_str = date_str[:4]
    try:
        return int(year_str)
    except ValueError:
        return None


def previous_and_next(some_iterable):  # pragma: no cover
    """taken from https://stackoverflow.com/a/1012089"""
    prevs, items, nexts = tee(some_iterable, 3)
    prevs = chain([None], prevs)
    nexts = chain(islice(nexts, 1, None), [None])
    return zip(prevs, items, nexts)


def normalize_string(string: str) -> str:
    """removese any superfluos whitespace from a given string"""
    return " ".join(" ".join(string.split()).split())


def make_entity_label(
    name_node: ET.Element, default_msg="no label provided", default_lang="en"
) -> tuple[str, str]:
    """Extracts a label and a lang tag from the past in name-node

    Args:
        name_node (ET.Element): A tei:persName|placeName|orgName element
        default_msg (str, optional): some default vaule for the label. Defaults to "no label provided".
        default_lang (str, optional): some default lang tag if the node does not provide and xml:lang attribute.
        Defaults to "en".

    Returns:
        tuple[str, str]: returns the extracted label and a lang tag
    """

    lang_tag = name_node.get("{http://www.w3.org/XML/1998/namespace}lang", default_lang)
    fornames = [
        normalize_string(x)
        for x in name_node.xpath(".//tei:forename//text()", namespaces=nsmap)
    ]
    surnames = [
        normalize_string(x)
        for x in name_node.xpath(".//tei:surname//text()", namespaces=nsmap)
    ]
    if len(surnames) > 0 and len(fornames) > 0:
        label = f"{surnames[0]}, {' '.join(fornames)}"
    elif len(surnames) == 0 and len(fornames) > 0:
        label = f"{' '.join(fornames)}"
    elif len(surnames) > 0 and len(fornames) == 0:
        label = f"{surnames[0]}"
    else:
        name_node_text = " ".join(name_node.xpath(".//text()", namespaces=nsmap))
        label = normalize_string(name_node_text)
    if label is None or label == "":
        label = default_msg
    return label, lang_tag


def make_bibl_label(
    node: ET.Element,
    no_author="o.A.",
    no_title="o.T.",
    year="o.J.",
    editor_abbr="(Hg.)",
    max_title_length=75,
) -> str:
    """creates a nice, bibliograhpically useful label from the passed in tei:biblStruct element

    Args:
        node (ET.Element): a tei:biblStruct element
        no_author (str, optional): Used if no author name can be extracted. Defaults to "o.A".
        no_title (str, optional): Used if no title can be extracted. Defaults to "o.T".
        year (str, optional): Used if no year can be extracted. Defaults to "o.J".
        editor_abbr(str, optional): how to mark the 'author' beeing an editor. Defaults to "(Hg.)".
        max_title_length(int, optional): max lenght for the title before it gets truncated. Defaults to

    Returns:
        str: _description_
    """
    try:
        author = node.xpath(".//tei:author[1]/tei:surname[1]", namespaces=nsmap)[0].text
    except IndexError:
        try:
            author = node.xpath(".//tei:author[1]/tei:name[1]", namespaces=nsmap)[
                0
            ].text
        except IndexError:
            try:
                author = node.xpath(
                    ".//tei:editor[1]/tei:surname[1]", namespaces=nsmap
                )[0].text
                author = f"{author} {editor_abbr}"
            except IndexError:
                try:
                    author = node.xpath(
                        ".//tei:editor[1]/tei:name[1]", namespaces=nsmap
                    )[0].text
                    author = f"{author} {editor_abbr}"
                except IndexError:
                    author = no_author
    try:
        year = node.xpath(".//tei:date[1]", namespaces=nsmap)[0].text
    except IndexError:
        year = year
    title = node.xpath(".//tei:title[1]", namespaces=nsmap)[0].text
    if title:
        if len(title) > max_title_length:
            title = f"{title[:max_title_length]}..."
    else:
        title = no_title
    return f"{author}, {title}, {year}"



import os
import unittest
from acdh_tei_pyutils.tei import TeiReader
from acdh_tei_pyutils.cli import mentions_to_indices


class MentionsToIndex(unittest.TestCase):
    """Tests for `acdh_tei_pyutils.mentions_to_index` function."""

    def setUp(self):
        """Set up test fixtures, if any."""
        print("setting up files")
        self.ref_event_text = "REF EVENT TEXT"
        self.mention_xpath = "//tei:rs/@ref"
        self.title_xpath = "//tei:title"
        self.test_path = "/tmp/acdh_pyutil_cli_test"
        self.editions_path = f"{self.test_path}/editions/"
        self.indices_path = f"{self.test_path}/indices/"
        os.makedirs(self.editions_path, exist_ok=True)
        os.makedirs(self.indices_path, exist_ok=True)
        edition_content = """
    <?xml version='1.0' encoding='UTF-8'?>
    <TEI xmlns="http://www.tei-c.org/ns/1.0" xml:base="." xml:id="fb_17000316_HanssGeorgWagner.xml">
        <body>
            <title>test document</title>
            <rs ref="#test_person">Maxi Muster</rs>
        </body>
    </TEI>
"""
        pers_index_content = """
    <?xml version='1.0' encoding='UTF-8'?>
    <TEI xmlns="http://www.tei-c.org/ns/1.0" xml:base="." xml:id="fb_17000316_HanssGeorgWagner.xml">
        <body>
            <listPerson>
                <person xml:id="test_person">
                    <persName source="#ignaza112">
                        <forename>
                            Maxi
                        </forename>
                        <surname>
                            Muster
                        </surname>
                    </persName>
                </person>
            </listPerson>
        </body>
    </TEI>
"""
        self.edition_doc = TeiReader(edition_content)
        self.person_index_doc = TeiReader(pers_index_content)
        self.edition_doc_path = self.editions_path + "text_document.xml"
        self.person_index_doc_path = self.indices_path + "persons.index"
        self.edition_doc.tree_to_file(self.edition_doc_path)
        self.edition_doc.tree_to_file(self.person_index_doc_path)

    def tearDown(self):
        """Tear down test fixtures, if any."""
        import shutil

        print("removing files")
        shutil.rmtree(self.test_path)

    def test_001_mentions_to_index(self):
        import click.testing

        runner = click.testing.CliRunner()
        args = f'-f "{self.edition_doc_path}" -i "{self.person_index_doc}"'
        args += f' -m "{self.mention_xpath}" -t "{self.ref_event_text}" -x "{self.title_xpath}"'
        result = runner.invoke(mentions_to_indices, args, catch_exceptions=False)
        print(result.output)
        print(result.exc_info)
        print(result.exception)
        assert result.exception is None
        assert result.exit_code == 0
